# Pack binary package files into an opk file
#
# Copyright (C) 2012 Patrick McDermott
#
# This file is part of opkbuild.
#
# opkbuild is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# opkbuild is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with opkbuild.  If not, see <http://www.gnu.org/licenses/>.

set -eu

build_opk()
{
	local binary="${1}"
	local version="${2}"
	local arch="${3}"
	local plat="${4}"
	local date="${5}"
	shift 5
	local find_not_link=
	local touch_noderef=

	ob_info "$(ob_get_msg 'build_opk')" \
		"${binary}_${version}_${arch}_${plat}.opk"

	# Don't dereference symbolic links.  They might be absolute paths, and
	# we don't want to attempt to affect the system on which we're building.
	# Also, we want to set the mtimes of the links themselves, if possible.
	if ${HAVE_TOUCH_NODEREF}; then
		find_not_link=''          touch_noderef='-h'
	else
		find_not_link='! -type l' touch_noderef=''
	fi
	find "${binary}.control" "${binary}.data" ${find_not_link} | xargs \
		${TOUCH} ${touch_noderef} -d "${date}"

	# This utility runs with (fake) privileges, so we can chown what we're
	# about to tar.
	find "${binary}.control" "${binary}.data" | xargs chown -h 0:0

	(cd -- "${binary}.control" && find '.' | LC_ALL=C sort | \
		${TAR} -cf '../control.tar' --no-recursion -T -)
	(cd -- "${binary}.data"    && find '.' | LC_ALL=C sort | \
		${TAR} -cf '../data.tar'    --no-recursion -T -)
	${TOUCH} -d "${date}" 'control.tar' 'data.tar'
	${GZIP} 'control.tar' 'data.tar'

	${TAR} -cf "../../${binary}_${version}_${arch}_${plat}.tar" \
		'debian-binary' 'control.tar.gz' 'data.tar.gz'
	rm -Rf 'control.tar.gz' 'data.tar.gz'
	${TOUCH} -d "${date}" "../../${binary}_${version}_${arch}_${plat}.tar"
	${GZIP} "../../${binary}_${version}_${arch}_${plat}.tar"
	mv "../../${binary}_${version}_${arch}_${plat}.tar.gz" \
		"../../${binary}_${version}_${arch}_${plat}.opk"

	return 0
}

main()
{
	local date=
	local pkg=
	local arch=
	local plat=

	if ! ob_set_text_domain 'opkbuild'; then
		printf '%s: Error: Failed to load locale messages\n' \
			"${0##*/}" >&2
		return 1
	fi

	ob_init_package '..' || return 1
	date="$(ob_iso8601_gmtime "${SOURCE_DATE_EPOCH}")"

	printf '2.0\n' >'debian-binary'
	${TOUCH} -d "${date}" 'debian-binary'
	if [ x"${OB_DO_SOURCE:+set}" = x'set' ]; then
		build_opk "src-${OPK_SOURCE}" "${OPK_SOURCE_VERSION}" \
			'src' 'all' "${date}"
	else
		for pkg in ${OPK_PACKAGES_REDUCED}; do
			arch="$(ob_get_binary_parameter "${pkg}" \
				'Architecture')"
			[ x"${arch}" != x'all' ] && arch="${OPK_HOST_ARCH}"
			plat="$(ob_get_binary_parameter "${pkg}" 'Platform')"
			[ x"${plat}" != x'all' ] && plat="${OPK_HOST_PLAT}"
			build_opk "${pkg}" "${OPK_BINARY_VERSION}" \
				"${arch}" "${plat}" "${date}"
		done
	fi
	rm 'debian-binary'

	return 0
}
